<?php

namespace App\Http\Controllers;

use App\AdminJournalVoucher;
use App\AdminOtherAccount;
use App\Company;
use App\Customer;
use App\Expense;
use App\Quotation;
use App\QuotationExpense;
use App\StaffAndSalesman;
use App\Supplier;
use App\OtherAccountBuyer;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class AdminJournalVoucherController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $journalVouchers = AdminJournalVoucher::with('account')->whereIn('id', function ($query) {
            $query->selectRaw('MIN(id)')
                ->from('admin_journal_vouchers')
                ->where('branch_id', session()->get('selected_branch'))
                ->whereNotNull('invoice_no')
                ->groupBy('invoice_no');
        })
        ->withTrashed()
        ->orderBy('id', 'desc')->get();

       

        return view('crusher.trans_journal_voucher.index', compact('journalVouchers'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $customers = Customer::where('branch_id', session()->get('selected_branch'))->get();
        $currentDate = Carbon::now()->format('Ymd');

        // Get the latest quotation number for the branch
        $latestVoucher = AdminJournalVoucher::where('branch_id', session()->get('selected_branch'))
            ->latest('id')->first();

        // Calculate the new quotation number
        if ($latestVoucher) {
            $lastJVNo = (int)substr($latestVoucher->invoice_no, -4); // Extract the last 4 digits
            $newJVNo = str_pad($lastJVNo + 1, 4, '0', STR_PAD_LEFT);
        } else {
            $newJVNo = '0001';
        }


        // Retrieve customers and include both 'account_name' and 'table_name' attributes
        $customers = Customer::where('branch_id', session()->get('selected_branch'))
        ->select('*', 'name_of_organisation as account_name')
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Customer');
        });

        // Retrieve suppliers and include both 'account_name' and 'table_name' attributes
        $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        ->select('*', 'name_of_organisation as account_name')
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Supplier');
        });

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
                $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        });

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });

        // Retrieve other accounts and set 'table_name' to the fully-qualified model class name
        $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        // Concatenate all collections into a single collection
        $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);


        return view('crusher.trans_journal_voucher.create', compact('customers', 'newJVNo', 'accounts'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        try {
            $validatedData = $request->validate([
                'invoice_no' => 'nullable|string',
                'date' => 'nullable|date',
                'account_id' => 'nullable|array',
                'account_id.*' => 'nullable',
                'account_type' => 'nullable|array',
                'account_type.*' => 'nullable',
                'narration' => 'nullable|array',
                'narration.*' => 'nullable|string',
                'type' => 'nullable|array',
                'type.*' => 'nullable|in:credit,debit',
                'amount' => 'nullable|array',
                'amount.*' => 'nullable',
                'name' => 'nullable|string',
                'phone' => 'nullable|string',
                'email' => 'nullable|string',
                'vat_number' => 'nullable|string',
                'address' => 'nullable|string',
                'type' => 'nullable|string',
            ]);
            $branch_id = session()->get('selected_branch');

              // Create an array to store buyer data
              $buyerData = [
                'name' => $validatedData['name'],
                'phone' => $validatedData['phone'],
                'email' => $validatedData['email'],
                'vat_number' => $validatedData['vat_number'],
                'address' => $validatedData['address'],
                'invoice_no' => $validatedData['invoice_no'],
                'type' => $validatedData['type'],
            ];

            // Check if buyer_account_id is present and create or update the buyer record
            if ($request->has('buyer_account_id')) {
                OtherAccountBuyer::updateOrCreate(['id' => $request->input('buyer_account_id')], $buyerData);
            } else {
                $buyer = OtherAccountBuyer::create($buyerData);
                $request->merge(['buyer_account_id' => $buyer->id]);
            }


            // Iterate over the item descriptions
            for ($i = 0; $i < count($validatedData['account_id']); $i++)
            {
                $data = [
                    'invoice_no' => $validatedData['invoice_no'],
                    'date' => $validatedData['date'],
                    'account_id' => $validatedData['account_id'][$i]??'',
                    'account_type' => $validatedData['account_type'][$i]??'',
                    'narration' => $validatedData['narration'][$i]??'',
                    'type' => $validatedData['type'][$i]??"",
                    'amount' => $validatedData['amount'][$i]??'',
                    'branch_id' => $branch_id,
                ];

                AdminJournalVoucher::create($data);
            }

            flash()->addSuccess('Record Created');
            return redirect()->route('crusher.journal_vouchers.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function show($journalVoucher)
    {
        $journalVoucher = AdminJournalVoucher::withTrashed()->find($journalVoucher);
        $journalVoucher = $journalVoucher->withTrashed()->with('account')->where('invoice_no', $journalVoucher->invoice_no)->get();
        $company = Company::where('user_id', auth()->id())->first();
        return view('crusher.trans_journal_voucher.view', compact('journalVoucher', 'company'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Quotation  $quotation
     * @return \Illuminate\Http\Response
     */
    public function edit(AdminJournalVoucher $journalVoucher)
    {
        $customers = Customer::where('branch_id', session()->get('selected_branch'))->get();
        $currentDate = Carbon::now()->format('Ymd');

        // Get the latest quotation number for the branch
        $latestVoucher = AdminJournalVoucher::where('branch_id', session()->get('selected_branch'))
            ->latest('id')->first();

        // Calculate the new quotation number
        if ($latestVoucher) {
            $lastJVNo = (int)substr($latestVoucher->invoice_no, -4); // Extract the last 4 digits
            $newJVNo = str_pad($lastJVNo + 1, 4, '0', STR_PAD_LEFT);
        } else {
            $newJVNo = '-0001';
        }

        // Retrieve customers and include both 'account_name' and 'table_name' attributes
        $customers = Customer::where('branch_id', session()->get('selected_branch'))
        ->select('*', 'name_of_organisation as account_name')
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Customer');
        });

        // Retrieve suppliers and include both 'account_name' and 'table_name' attributes
        $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        ->select('*', 'name_of_organisation as account_name')
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Supplier');
        });

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
                $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        });

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });

        // Retrieve other accounts and set 'table_name' to the fully-qualified model class name
        $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        // Concatenate all collections into a single collection
        $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);
;

        $journalVouchers = $journalVoucher->with('account')->where('invoice_no', $journalVoucher->invoice_no)->get();

        return view('crusher.trans_journal_voucher.edit', compact('accounts', 'journalVouchers'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Quotation  $quotation
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, AdminJournalVoucher $journalVoucher)
    {
        try {
            $validatedData = $request->validate([
                'invoice_no' => 'nullable|string',
                'date' => 'nullable|date',
                'account_id' => 'nullable|array',
                'account_id.*' => 'nullable',
                'account_type' => 'nullable|array',
                'account_type.*' => 'nullable',
                'narration' => 'nullable|array',
                'narration.*' => 'nullable|string',
                'type' => 'nullable|array',
                'type.*' => 'nullable|in:credit,debit',
                'amount' => 'nullable|array',
                'amount.*' => 'nullable',
            ]);
            $branch_id = session()->get('selected_branch');
            $journalVoucher = $journalVoucher->with('account')->where('invoice_no', $journalVoucher->invoice_no)->forceDelete();
            // Iterate over the item descriptions
            for ($i = 0; $i < count($validatedData['account_id']); $i++)
            {
                $data = [
                    'invoice_no' => $validatedData['invoice_no'],
                    'date' => $validatedData['date'],
                    'account_id' => $validatedData['account_id'][$i]??'',
                    'account_type' => $validatedData['account_type'][$i]??'',
                    'narration' => $validatedData['narration'][$i]??'',
                    'type' => $validatedData['type'][$i]??"",
                    'amount' => $validatedData['amount'][$i]??'',
                    'branch_id' => $branch_id,
                ];

                AdminJournalVoucher::create($data);
            }

            flash()->addSuccess('Record Updated');
            return redirect()->route('crusher.journal_vouchers.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Quotation  $quotation
     * @return \Illuminate\Http\Response
     */
    public function destroy(Quotation $quotation)
    {
        //
    }

    public function softDelete($journalVoucher)
    {
        $journalVoucher = AdminJournalVoucher::findOrFail($journalVoucher);
        $journalVoucher->where('invoice_no', $journalVoucher->invoice_no)->delete();
        flash()->addSuccess('Success');
        return redirect()->route('crusher.journal_vouchers.index');
    }
}
